#title="賢く改行"
#include "kili/KILI.js"
#include "kili/StringEx.js"
#include "kili/ArrayEx.js"
////////////////////////////////////////////////////////////////////////////////
//■概要:
//ちょっと賢く改行する。
//■機能:
//・ブロック開始記号の直後で改行した場合にブロック（次行インデント+閉じ括弧）を作成する。選択範囲直後がブロック終了記号だった場合はそれを利用する。
//	・ブロックを作る場合、選択部分は次行へ。
//	・ブロックを作らない場合、選択部分は削除。（通常の改行と同じ）
//	・選択範囲以降の文字列の位置は指定する。
//・行コメント内の場合、次行も行コメントにする。
//■使い方:
//「//SmartBreak();」という行の「//」を削除。
////////////////////////////////////////////////////////////////////////////////
//@mode:<string> 編集モード。定義が存在しない場合はDEFAULTを適用する。*:自動選択。
//@exception:<number> 改行時に「追加しない」もの。1:行コメント、2:ブロック、4:行コメント中のブロック、8:元々のインデント、16:ブロック内のインデント。
//@push:<number> 選択範囲以降の文字列の処理。-1:そのまま、*0:次行に追加、1:最後に追加、2:削除。
var SmartBreak=function(mode,exception,push){
	var $s=GSO();
	if($s.GetActivePointX(mePosLogical)==1){
		$s.Text="\n";
		return;
	}
	//編集モードごとの設定
	//モード名は大文字にすること。
	//@comment:<Array> コメント文字列。
	//@bracket:<Object{open:close,...}> ブロックの開始と終了の文字列。指定しなければブロック化しない。
	//@indent:<string> インデントに使う文字列。*:\t。
	var modes={
		BAT:{comment:["rem","::"]},
		COLDFUSION:{comment:["//"]},
		DELPHI:{comment:["//"]},
		HSP:{comment:[";"]},
		INI:{comment:[";","#"]},
		PERL:{comment:["#"]},
		PHP:{comment:["//","#"]},
		POWERSHELL:{comment:["#"]},
		PYTHON:{comment:["#"]},
		RUBY:{comment:["#"]},
		SQL:{comment:["--"]},
		TEX:{comment:["%"]},
		UWSC:{comment:["//"]},
		VBSCRIPT:{comment:["'"]},
		VISUALBASIC:{comment:["'"]},
		"X86 ASSEMBLER":{comment:[";"]},
		DEFAULT:{comment:["//"],bracket:{"{":"}"}}
	};
	mode=mode||modes.DEFAULT;
	if(typeof(mode)!="object"){
		mode=modes[mode.toUpperCase()]||modes.DEFAULT;
	}
	var comment=mode.comment||[];
	var bracket=mode.bracket||{};
	var left=KILI.selex.getLeft();
	var right=KILI.selex.getRight();
	var sel=$s.Text;
	var originalIndent=/^\s+/.test(left)?RegExp.lastMatch:"";
	for(var i=0;i<comment.length;i++){
		comment[i]=comment[i].escapeRegExp();
	}
	var lineComment=left.startsWith(new RegExp(originalIndent+comment.join("|").enclose("(",")")));
	comment=RegExp.$1;
	var open=[];
	for(var c in bracket){
		open.push(c.escapeRegExp());
	}
	var blockStart=left.endsWith(new RegExp(open.join("|").enclose("(?:",")")));
	var close=blockStart?bracket[RegExp.lastMatch]:"";
	var blockEnd=blockStart&&right.startsWith(new RegExp(close.escapeRegExp()));
	var seg=[""];//[0]はキャレット行になる
	var addComment=!(exception&1);
	var addBlock=!(exception&2);
	var addBlockInComment=addComment&&addBlock&&!(exception&4);//行コメント中のブロックは、行コメントとブロックを作成する場合のみ有効
	var addOriginalIndent=!(exception&8);
	var indent=exception&16?"":mode.indent||"\t";
	var delta=1;
	if(addBlock&&blockStart&&(addBlockInComment||!lineComment)){
		seg.push(indent+sel);
		delta+=indent.length;
		if(blockEnd){//選択範囲直後がブロック終了記号ならそれを利用する
			right=right.slice(close.length);
		}
		if(push==0&&right.length){
			if(sel==""){//無選択の場合、ブロックの中身は選択範囲の右のみにする
				seg[seg.length-1]=indent+right;
			}
			else{
				seg.push(indent+right);
			}
		}
		else if(push==-1){
			seg[0]=right;
			delta+=right.length;
		}
		seg.push(close+(push==1?right:""));
	}
	else{//ブロックを作らない場合、選択範囲は削除
		seg.push(right);
	}
	if(addComment&&lineComment){
		for(var i=1;i<seg.length;i++){
			seg[i]=comment+seg[i];
		}
		delta+=comment.length;
	}
	if(addOriginalIndent){
		for(var i=1;i<seg.length;i++){
			seg[i]=originalIndent+seg[i];
		}
		delta+=originalIndent.length;
	}
	var top=KILI.selex.getTopPos();
	$s.SetActivePos(top);
	$s.EndOfLine(true,mePosLogical);
	$s.Text=seg.join("\n");
	$s.SetActivePos(delta+top);
}
//SmartBreak();
////////////////////////////////////////////////////////////////////////////////
//■変更:
//全て作り直し。どういう時にどの様に動作するかを明確に。
//キャレット位置ではなく、選択範囲を基準に。
//--------------------------------------
